/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl.compiler;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.util.Printer;

/**
 * Abstract compiler builder.
 */
public abstract class AbstractBuilder implements BuilderAttributes {

  // --------------------------------------------------------------------------
  // Attribute Fields
  // --------------------------------------------------------------------------
  private String flags;

  // --------------------------------------------------------------------------
  // Implementation of the AttributeController interface
  // --------------------------------------------------------------------------

  public String getFlags() {
    return flags;
  }

  public void setFlags(final String flags) {
    this.flags = flags;
  }

  // --------------------------------------------------------------------------
  // Utility methods
  // --------------------------------------------------------------------------

  protected static String getContextArg(final String name,
      final Map<Object, Object> context) throws ADLException {
    final String s = tryGetContextArg(name, context);
    if (s == null)
      throw new ADLException("Can't find contextual argument \'" + name
          + "\' in the context.");
    return s;
  }

  protected static String tryGetContextArg(final String name,
      final Map<Object, Object> context) {
    return (String) context.get(name);
  }

  protected static class CommandLine {
    private String             executable = null;
    private final StringBuffer args       = new StringBuffer(" ");

    protected void setExecutable(final String command) {
      executable = command;
    }

    protected void createArgument(final String arg) {
      args.append(arg).append(' ');
    }

    protected String getCommandLine() {
      return executable + args.toString();
    }
  }

  protected static void printCompilerMessages(final Process p)
      throws InterruptedException {
    final PrinterThread out = new PrinterThread(p.getInputStream());
    final PrinterThread err = new PrinterThread(p.getErrorStream());
    out.start();
    err.start();
    out.join();
    err.join();
  }

  private static class PrinterThread extends Thread {

    private final LineNumberReader reader;

    protected PrinterThread(final InputStream s) {
      reader = new LineNumberReader(new InputStreamReader(
          new BufferedInputStream(s)));
    }

    @Override
    public void run() {
      String line;
      try {
        while ((line = reader.readLine()) != null) {
          Printer.error(line);
        }
        reader.close();
      } catch (final IOException e) {
        throw new InternalError(e.getMessage());
      }
    }

  }
}
