/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl.idl;

import java.util.Map;

import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.cecilia.adl.idl.ast.IDLDefinition;
import org.objectweb.fractal.task.codegen.lib.AbstractSourceFileProviderTask;

/**
 * Abstract IDL compiler.
 */
public abstract class AbstractIDLCompiler
    implements
      BindingController,
      IDLCompilerAttributes,
      IDLCompiler {

  // --------------------------------------------------------------------------
  // Attribute
  // --------------------------------------------------------------------------

  protected String           taskSuffix;

  // --------------------------------------------------------------------------
  // Client interfaces
  // --------------------------------------------------------------------------

  /** The name of the builder interface. */
  public static final String BUILDER_BINDING = "builder";
  /** The builders client interfaces. */
  public IDLBuilder          buildersItf;

  // --------------------------------------------------------------------------
  // Task class
  // --------------------------------------------------------------------------

  protected static class IDLDefinitionTask
      extends
        AbstractSourceFileProviderTask {

    private final IDLDefinition idlDefinition;
    private final IDLBuilder    builder;

    protected IDLDefinitionTask(final IDLDefinition idlDefinition,
        final IDLBuilder builder) {
      this.idlDefinition = idlDefinition;
      this.builder = builder;
    }

    public void execute(final Map<Object, Object> context) throws Exception {
      final Object componentType = (getFactoryProviderTask() == null)
          ? null
          : getFactoryProviderTask().getFactory();
      final Object componentInstance = (getInstanceProviderTask() == null)
          ? null
          : getInstanceProviderTask().getInstance();
      builder.createIDLDefinition(componentType, componentInstance,
          idlDefinition, getCodePieces(), context);
    }
  }

  // ------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ------------------------------------------------------------------------

  public String[] listFc() {
    return new String[]{BUILDER_BINDING};
  }

  public Object lookupFc(final String itf) {
    if (itf.equals(BUILDER_BINDING)) {
      return buildersItf;
    }
    return null;
  }

  public void bindFc(final String itf, final Object value) {
    if (itf.equals(BUILDER_BINDING)) {
      buildersItf = (IDLBuilder) value;
    }
  }

  public void unbindFc(final String itf) {
    if (itf.equals(BUILDER_BINDING)) {
      buildersItf = null;
    }
  }

  // ------------------------------------------------------------------------
  // Implementation of the AttributeController interface
  // ------------------------------------------------------------------------

  public String getTaskSuffix() {
    return taskSuffix;
  }

  public void setTaskSuffix(final String taskSuffix) {
    this.taskSuffix = taskSuffix;
  }
}
