/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.idl;

import java.util.Map;

import org.objectweb.fractal.cecilia.adl.idl.ast.IDLDefinition;

/**
 * Creates IDL Definition.
 */
public interface IDLBuilder {

  /**
   * Creates header files or stub'skeletons for complex bindings for the given
   * IDL definition.
   * 
   * @param componentType the type of the component. This parameter can be null
   *            if the implementation of this interface is generic for all
   *            components.
   * @param componentInstance the instance of the component. This parameter can
   *            be null if the implementation of this interface is generic for
   *            all components.
   * @param additionalInformation Additional informations that will be used by
   *            the implementation of this interface. This parameter can be null
   *            if no additional information is needed.
   * @param idlDefinition the IDL definition to be used.
   * @param codePieces the code pieces that has been produced by builders this
   *            builder depends on.
   * @param context additional parameters.
   * @return the code produced by the implementor if this interface.
   * @throws Exception if something wrong happens
   */
  Object createIDLDefinition(Object componentType, Object componentInstance,
      IDLDefinition idlDefinition, Map<String, Object> codePieces,
      Map<Object, Object> context) throws Exception;
}
