/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl.interfaces;

import java.util.Map;

import org.objectweb.fractal.adl.types.TypeInterface;

/**
 * A builder interface to create a component interfaces.
 */
public interface InterfaceBuilder {

  /**
   * Creates the interface of the component.
   * 
   * @param componentType the type of the component.
   * @param componentInstance the component instance.
   * @param contentType the type of the content part of the component.
   * @param interfaceTypes the types of the interfaces.
   * @param codePieces the code pieces that has been produced by builders this
   *          builder depends on.
   * @param context additional parameters.
   * @return the code produced by this builder.
   * @throws Exception if something wrong append.
   */
  Object createInterfaces(Object componentType, Object componentInstance,
      Object contentType, TypeInterface[] interfaceTypes,
      Map<String, Object> codePieces, Map<Object, Object> context)
      throws Exception;
}
