/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.attributes;

import java.util.Map;

import org.objectweb.fractal.adl.attributes.Attribute;
import org.objectweb.fractal.cecilia.adl.attributes.AttributeBuilder;
import org.objectweb.fractal.cecilia.adl.idl.ast.Field;
import org.objectweb.fractal.cecilia.adl.idl.ast.FieldContainer;
import org.objectweb.fractal.cecilia.adl.idl.ast.Type;
import org.objectweb.fractal.cecilia.adl.idl.util.CUtil;
import org.objectweb.fractal.cecilia.adl.idl.util.Util;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the initialization of the attributes of a component written in
 * <code>thinkMC</code> languages.
 */
public class AttributeInstanceBuilder implements AttributeBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the AttributesBuilder interface
  // --------------------------------------------------------------------------

  public Object setAttributes(final Object componentType,
      final Object componentInstance, final Object attributeAST,
      final Attribute[] fields, final Map<String, Object> codePieces,
      final Map<Object, Object> context) {
    final CodeWriter cw = new CodeWriter();
    cw.append("{");
    for (final Attribute field : fields) {
      // append cast to actual attribute type.
      cw.append('(').append(
          CUtil.buildComplexOrPrimitiveType(getAttributeType(attributeAST,
              field.getName()))).append(") ");
      final String valSym = field.getValue();
      try {
        Integer.parseInt(valSym);
        cw.append(valSym).append(", ");
      } catch (final NumberFormatException e) {
        if (valSym.startsWith("0x") || valSym.startsWith("0X"))
          cw.append(valSym).append(", ");
        else
          cw.append("\"").append(valSym).append("\", ");
      }
    }
    cw.append("}");
    return cw.toString();
  }

  protected Type getAttributeType(final Object attributeAST, final String name) {
    for (final Field field : ((FieldContainer) attributeAST).getFields()) {
      if (field.getName().equals(name)) {
        return Util.getContainedType(field);
      }
    }
    return null;
  }
}
