/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.components;

import java.util.Map;

import org.objectweb.fractal.cecilia.adl.components.ComponentBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the definition of a component written in <code>thinkMC</code>
 * language.
 */
public class ComponentDefinitionBuilder implements ComponentBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the ComponentBuilder interface
  // --------------------------------------------------------------------------

  public Object createComponent(final Object componentType,
      final Object componentInstance, final Object controllerDesc,
      final Object contentDesc, final Map<String, Object> codePieces,
      final Map<Object, Object> context) throws Exception {
    final CodeWriter cw = new CodeWriter("Component Definition Builder");

    final String componentCName = componentType.toString().replace('.', '_');

    final String interfaces = (String) codePieces.get("interfaces");
    cw.appendln(interfaces);

    final String attributes = (String) codePieces.get("attributes");
    cw.appendln(attributes);

    cw.append("#define __CECILIA__DEFINE_COMPONENT_TYPE \\").endl();
    cw.append("struct ").append(componentCName).append("_t { \\").endl();

    if (interfaces != null)
      cw.append("struct ").append(componentCName).append("_type type; \\")
          .endl();

    if (attributes != null)
      cw.append(componentCName).append("_attributes_t attributes; \\").endl();

    cw.append("  struct ").append(componentCName).append(
        "_instancedata data ; \\").endl();
    cw.append("} ;").endl();

    cw.endl();
    cw.appendln((String) codePieces.get("interfaces.vft"));
    cw.appendln((String) codePieces.get("controller"));

    return cw.toString();
  }

}
