/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.source;

import static org.objectweb.fractal.cecilia.adl.types.TypeDecorationUtil.getTypeDecoration;

import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.CompilerError;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.interfaces.Interface;
import org.objectweb.fractal.adl.interfaces.InterfaceContainer;
import org.objectweb.fractal.adl.types.TypeInterfaceUtil;
import org.objectweb.fractal.cecilia.adl.source.AbstractSourceFileCompiler;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * Add a task for the generation of a file containing the definition of the
 * fractal type of a component. This compiler should be invoked only on
 * component that defines a type (i.e. on component such as
 * <code>{@link org.objectweb.fractal.cecilia.adl.types.TypeDecorationUtil}.{@link org.objectweb.fractal.cecilia.adl.types.TypeDecorationUtil#getTypeDecoration(ComponentContainer) getTypeDecoration}(component) == component</code>).
 */
public class TypeSourceFileCompiler extends AbstractSourceFileCompiler {

  // --------------------------------------------------------------------------
  // Implementation of the PrimitiveCompiler interface
  // --------------------------------------------------------------------------

  public void compile(final List<ComponentContainer> path,
      final ComponentContainer container, final TaskMap tasks,
      final Map<Object, Object> context) throws ADLException {

    if (getTypeDecoration(container) != container) {
      throw new CompilerError(
          "This compiler must be invoked only on component that defines a type.");
    }

    if (!(container instanceof InterfaceContainer)) return;

    // Checks that the component has at least one server interface
    boolean hasServerItf = false;
    for (final Interface itf : ((InterfaceContainer) container).getInterfaces()) {
      if (TypeInterfaceUtil.isServer(itf)) {
        hasServerItf = true;
        break;
      }
    }
    if (!hasServerItf) return;

    final SourceFileTask sourceTask = new SourceFileTask(builderItf);
    tasks.addTask("type-header-file", container, sourceTask);
    sourceTask.setFactoryProviderTask(tasks.getTaskHole("type", container));
    sourceTask.addSourceCodeProviderTask("exported-interfaces.definition",
        tasks.getTaskHole("exported-interfaces.definition", container));
  }
}
