/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.types;

import static org.objectweb.fractal.cecilia.adl.controllers.ControllerDecorationUtil.getVTable;

import java.io.File;
import java.util.Map;

import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.cecilia.adl.idl.util.CUtil;
import org.objectweb.fractal.cecilia.adl.interfaces.InterfaceBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the definition of the exported interfaces of a component written in
 * <code>thinkMC</code> language.
 */
public class ExportedDefinitionBuilder implements InterfaceBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the InterfaceBuilder interface
  // --------------------------------------------------------------------------

  public Object createInterfaces(final Object componentType,
      final Object componentInstance, final Object contentType,
      final TypeInterface[] interfaceTypes,
      final Map<String, Object> codePieces, final Map<Object, Object> context) {
    final String name = componentType.toString().replace('.', '_');

    final CodeWriter cw = new CodeWriter(
        "Exported Interface Definition Builder");
    // include idl.h files
    for (final TypeInterface itf : interfaceTypes)
      cw.append(CUtil.buildInclude(itf.getSignature()));
    cw.endl();

    // Build the structure of provided interfaces
    cw.append("struct ").append(componentType.toString().replace('.', '_'))
        .append("_exporteds {").endl();
    for (final TypeInterface itf : interfaceTypes) {
      cw.append("R").append(itf.getSignature().replace('.', '_')).append(" ")
          .append(itf.getName().replace('-', '_')).append(";").endl();
    }
    cw.append("};").endl();
    cw.endl();

    // build the definition of the methods implemented by the component.
    for (final TypeInterface itf : interfaceTypes) {
      if (getVTable(itf) == null) {
        final String signature = itf.getSignature().replace('.', '_');
        final String itfName = itf.getName();
        final String cItfName = itfName.replace('-', '_');

        // include VFTTemplate header file and use the <itf>_METHOD_DEFINITION
        // macro to define the methods implemented by the component.
        cw.append("// defines the method prototype implemented by this ")
            .append("component for the interface ").append(itfName).endl();
        cw.append("#include \"").append(
            itf.getSignature().replace('.', File.separatorChar)).append(
            "VFTTemplate.h\"").endl();
        cw.append(signature.toUpperCase()).append("_METHOD_DEFINITION(")
            .append(name).append(",").append(cItfName).append(") ;").endl();
        cw.endl();
      }
    }
    cw.endl();

    return cw.toString();
  }
}
