/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.types;

import static org.objectweb.fractal.cecilia.adl.controllers.ControllerDecorationUtil.getDataSuffix;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.cecilia.adl.interfaces.InterfaceBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds a part of the implementation of the <code>factory</code> interface
 * related to the initialization of the interfaces exported by the component.
 * <br>
 * This builder produces one code piece
 * <ul>
 * <li><code>"instantiate.implem"</code> containing the actual part of the
 * implementation of the <code>instantiate</code> method.</li>
 * </ul>
 * The generated code supposes that the original component is pointed by a
 * variable whose name is <code>originalComp</code> and the component being
 * instantiated is pointed by a variable whose name is <code>newComp</code>.
 * Moreover, controller structures must be pointed by variables whose name are
 * <code>newComp_<i>CTRL_DATA</i></code> where <code><i>CTRL_DATA</i></code>
 * is the value of the <code>"data"</code> decoration of the controller
 * interface.
 * 
 * @see org.objectweb.fractal.cecilia.primitive.thinkMC.components.FactoryImplementationBuilder
 * @see org.objectweb.fractal.cecilia.primitive.thinkMC.controllers.ACFactoryBuilder
 * @see org.objectweb.fractal.cecilia.primitive.thinkMC.controllers.BCPrimitiveFactoryBuilder
 * @see org.objectweb.fractal.cecilia.primitive.thinkMC.controllers.CIFactoryBuilder
 */
public class ExportedFactoryBuilder implements InterfaceBuilder {

  public Object createInterfaces(final Object componentType,
      final Object componentInstance, final Object contentType,
      final TypeInterface[] interfaceTypes,
      final Map<String, Object> codePieces, final Map<Object, Object> context)
      throws Exception {

    final Map<String, String> producedCode = new HashMap<String, String>();
    final CodeWriter cw = new CodeWriter("Exported Interfaces Factory Builder");

    for (final TypeInterface itf : interfaceTypes) {
      final String data = getDataSuffix(itf);
      cw.append("newComp->type.exported.").append(
          itf.getName().replace('.', '_').replace('-', '_')).append(
          ".selfdata = ");
      if (data == null) {
        // data of this interface is the component data
        cw.append("newComp;");
      } else {
        // data of this interface is controller data
        cw.append("newComp_").append(data).append(";");
      }
      cw.endl();
    }

    producedCode.put("instantiate.implem", cw.toString());
    return producedCode;
  }
}
