/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.types;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.interfaces.Interface;
import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.cecilia.adl.interfaces.InterfaceBuilder;
import org.objectweb.fractal.cecilia.adl.interfaces.InterfaceDecorationUtil;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the instantiation of the imported interfaces of a component written in
 * <code>thinkMC</code> language.
 */
public class ImportedInstanceBuilder implements InterfaceBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the InterfaceBuilder interface
  // --------------------------------------------------------------------------

  public Object createInterfaces(final Object componentType,
      final Object componentInstance, final Object contentType,
      final TypeInterface[] interfaceTypes,
      final Map<String, Object> codePieces, final Map<Object, Object> context) {
    final Map<String, String> producedCode = new HashMap<String, String>();
    CodeWriter cw = new CodeWriter(
        "Imported Interfaces Instantiation Builder, server components");
    for (final TypeInterface element : interfaceTypes) {
      final Node itf = (Node) element;
      Object serverType = null;
      Object serverInstance = null;
      if (itf
          .astGetDecoration(InterfaceDecorationUtil.NO_STATIC_BINDING_DECORATION) == null) {
        serverType = itf.astGetDecoration("server-type");
        serverInstance = itf.astGetDecoration("server-instance");
      }
      if (serverType == null || serverInstance == null) continue;

      // Bug 307360: if the server component has the same type as this component
      // do not include ".adl.h"
      final String instanceName = serverInstance.toString().replace('.', '_');
      final String typeName = serverType.toString().replace('.', '_');
      if (serverType == componentType) {
        cw.append("#ifndef ").append(instanceName).append("_DECLARED").endl();
        cw.append("extern struct ").append(typeName).append("_t ").append(
            instanceName).append(";").endl();
        cw.append("#define ").append(instanceName).append("_DECLARED").endl();
        cw.appendln("#endif");
      } else {
        cw.append("#ifndef ").append(instanceName).append("_DECLARED").endl();
        cw.append("#include \"").append(
            serverType.toString().replace('.', File.separatorChar)).append(
            ".adl.h\"").endl();
        cw.append("extern struct ").append(typeName).append("_exporteds ")
            .append(instanceName).append(";").endl();
        cw.append("#define ").append(instanceName).append("_DECLARED").endl();
        cw.appendln("#endif");
      }
    }
    producedCode.put("include", cw.toString());

    cw = new CodeWriter(
        "Imported Interfaces Instantiation Builder, client interfaces initialization");
    cw.append("{").endl();
    for (final TypeInterface clientItf : interfaceTypes) {
      final Node itf = (Node) clientItf;
      Object serverType = null;
      Object serverInstance = null;
      Interface serverItf = null;
      if (itf
          .astGetDecoration(InterfaceDecorationUtil.NO_STATIC_BINDING_DECORATION) == null) {
        serverType = itf.astGetDecoration("server-type");
        serverInstance = itf.astGetDecoration("server-instance");
        serverItf = (Interface) itf.astGetDecoration("bound-to");
      }
      if (serverInstance == null || serverType == null || serverItf == null)
        cw.append("0");
      else if (serverType == componentType)
        cw.append("(R" + clientItf.getSignature().replace('.', '_') + "*)&(")
            .append(serverInstance.toString().replace('.', '_')).append(
                ".type.exported.")
            .append(serverItf.getName().replace('-', '_')).append(')');
      else
        cw.append("(R" + clientItf.getSignature().replace('.', '_') + "*)&(")
            .append(serverInstance.toString().replace('.', '_')).append('.')
            .append(serverItf.getName().replace('-', '_')).append(')');

      cw.append(", // ").append(clientItf.getName()).endl();

    }
    cw.append("}").endl();

    producedCode.put("init", cw.toString());

    return producedCode;
  }
}
