/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.types;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.cecilia.adl.interfaces.InterfaceBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the instantiation of the interfaces of a component written in
 * <code>thinkMC</code> language.
 */
public class InterfaceInstanceBuilder implements InterfaceBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the InterfaceBuilder interface
  // --------------------------------------------------------------------------

  public Object createInterfaces(final Object componentType,
      final Object componentInstance, final Object contentType,
      final TypeInterface[] interfaceTypes,
      final Map<String, Object> codePieces, final Map<Object, Object> context) {
    final Map<String, String> producedCode = new HashMap<String, String>();
    CodeWriter cw = new CodeWriter();
    cw.appendln((String) codePieces.get("exported-interfaces.include"));
    cw.appendln((String) codePieces.get("imported-interfaces.include"));
    producedCode.put("include", cw.toString());

    cw = new CodeWriter("Interfaces Instantiation Builder");
    cw.append("{").endl();
    final String exporteds = (String) codePieces
        .get("exported-interfaces.init");
    if (exporteds != null) cw.append(exporteds);
    cw.append(',').endl();
    final String importeds = (String) codePieces
        .get("imported-interfaces.init");
    if (importeds != null) cw.append(importeds);
    cw.append("}").endl();

    producedCode.put("init", cw.toString());

    return producedCode;
  }

}
